if (Test-Path ".\tempconfig.json"){
  $settings = (Get-Content ".\tempconfig.json" -Raw) | ConvertFrom-Json
  $constants = (Get-Content ".\scripts\settings.json" -Raw) | ConvertFrom-Json
  $settings.IISAuthentication = ($settings.IISAuthentication).ToLower()
}
elseif (Test-Path ".\config.json"){
  $Docker = $false
  $settings = (Get-Content ".\config.json" -Raw) | ConvertFrom-Json
  $constants = (Get-Content ".\scripts\settings.json" -Raw) | ConvertFrom-Json
  $settings.IISAuthentication = ($settings.IISAuthentication).ToLower()
}
else{
  $Docker = $true
}
$ErrorActionPreference = "Stop"
$WarningPreference = "SilentlyContinue"

$pwd = (Get-Item -Path ".\" -Verbose).FullName
$package = $pwd + "\package"

#Download and install Node.js
$NPM_CONFIG_LOGLEVEL = "info"
$NODE_VERSION = $constants.NodeJSVersion
$NODE_SHA256 = $constants.NodeJSSHA256
$NODE_VERSION_FULL = "v" + $NODE_VERSION
$NODE_URL = $constants.NodeJSURL
$NodeVersionInstalled = ""
$NodeJSPath = $settings.NodeJSPath

try{
  $NodeVersionInstalled = node -v
}
catch{}


if ($NodeVersionInstalled -eq $null -Or $NodeVersionInstalled -eq ""){
  $message = "7.  Installing Node.js..."
  $message | Out-File -Append log.txt
  Write-Host -NoNewline $message
  if ((Get-FileHash "$package\node.zip" -Algorithm sha256).Hash -ne $NODE_SHA256) {exit 1}
  (New-object -com shell.application).namespace($NodeJSPath).CopyHere((New-object -com shell.application).namespace("$package\node.zip").Items(),20);
  if (!(Test-Path $NodeJSPath\nodejs)){
    Rename-Item -Path $("$NodeJSPath\node-v{0}-win-x64" -f $NODE_VERSION) -NewName "$NodeJSPath\nodejs" | Out-File -Append log.txt
  }
  if (!(Test-Path($env:APPDATA + '\npm'))){
    New-Item $($env:APPDATA + '\npm') -Type directory | Out-File -Append log.txt
  }
  $env:PATH = "$NodeJSPath\nodejs;{0}\npm;{1}" -f $env:APPDATA, $env:PATH
  [Environment]::SetEnvironmentVariable('PATH', $env:PATH, [EnvironmentVariableTarget]::Machine)
  Write-Host "`r$message Done."
}
elseif ($NodeVersionInstalled -ne $NODE_VERSION_FULL){
  $message = "7.  Node.js is already installed with a different version number than $NODE_VERSION_FULL."
  $message | Out-File -Append log.txt
  Write-Host $message
}
else{
  $message = "7.  Node.js is already installed."
  $message | Out-File -Append log.txt
  Write-Host $message
}

$message = "8.  Installing .NET and IIS Windows features."
$message | Out-File -Append log.txt
Write-Host $message
$featuresCollection = "IIS-WebServerRole", "IIS-ASPNET45", "IIS-WebServerManagementTools", "IIS-BasicAuthentication", "IIS-WindowsAuthentication"
foreach ($feature in $featuresCollection){
  if((Get-WindowsOptionalFeature -FeatureName  $feature -Online).State -eq "Enabled") {
    $message = "    $feature is already installed."
    $message | Out-File -Append log.txt
    Write-Host $message
  }
  else {
    $message = "    Installing $feature..."
    $message | Out-File -Append log.txt
    Write-Host -NoNewLine $message
    if($feature -eq "IIS-WebServerRole"){
      Enable-WindowsOptionalFeature -online -featurename $feature | Out-File -Append log.txt
    }
    else{
      Enable-WindowsOptionalFeature -online -featurename $feature -all | Out-File -Append log.txt
    }
    Write-Host "`r$message Done."
  }
}
Write-Host "    Done."

$EngineService = (Get-WmiObject -Class Win32_Service -Filter "Name='WorkflowGenEngineService'")
if ($EngineService -ne $null -And $EngineService.State -eq "Running"){
  Write-Host "    Stopping WorkflowGen Engine Service."
  $EngineService.StopService() | Out-File -Append log.txt
}

$DirService = (Get-WmiObject -Class Win32_Service -Filter "Name='WorkflowGenDirSyncService'")
if ($DirService -ne $null -And $DirService.State -eq "Running"){
  Write-Host "    Stopping WorkflowGen Directory Sync Service."
  $DirService.StopService() | Out-File -Append log.txt
}

$message = "9.  Copying WorkflowGen website and services files..."
$message | Out-File -Append log.txt
Write-Host -NoNewLine $message
$WFGWebAppPathRoot = Split-Path -Path $settings.WebAppPath
$WFGWebAppPathLeaf = Split-Path -Path $settings.WebAppPath -Leaf

if (!(Test-Path $settings.WebAppPath)){
  New-Item  $settings.WebAppPath -ItemType Directory | Out-Null
}
Copy-Item -Path ($package+"\manual\Inetpub\wwwroot\wfgen\*") -Destination $settings.WebAppPath -Recurse -force

if (!(Test-Path $settings.ServicesPath)){
  New-Item  $settings.ServicesPath -ItemType Directory | Out-Null
}
Copy-Item -Path ($package+"\manual\Program Files\Advantys\WorkflowGen\Services\*") -Destination $settings.ServicesPath -Recurse -force

if ($Docker -eq $true){
  Copy-Item -Path ($package+"\web.config") -Destination $settings.WebAppPath -force
  Copy-Item -Path ($package+"\WFG-TRIAL.lic") -Destination ($settings.WebAppPath+"\bin") -Recurse -force
}
if ($settings.LicenseType -eq "trial"){
  Copy-Item -Path ($package+"\WFG-TRIAL.lic") -Destination ($settings.WebAppPath+"\bin") -Recurse -force
}
Write-Host "`r$message Done."

#Create the windows user for Basic/Windows auth
if ($settings.IISAuthentication -ne "workflowgen"){
  $filter = "LocalAccount='true' and Name='"+$settings.WFGAdminUsername+"'"
  if ((Get-WmiObject Win32_UserAccount -Filter $filter) -eq $null){
    $message = "10. Creating Windows account: " + $settings.WFGAdminUsername + "..."
    $message | Out-File -Append log.txt
    Write-Host -NoNewLine $message
    $group = "Administrators"
    NET USER $settings.WFGAdminUsername $settings.WFGAdminPassword /add /y /expires:never | Out-File -Append log.txt
    NET LOCALGROUP $group $settings.WFGAdminUsername /add | Out-File -Append log.txt
    Write-Host "`r$message Done."
  }
  else{
    $message = "10. Windows account already exists: " + $settings.WFGAdminUsername
    $message | Out-File -Append log.txt
    Write-Host $message
  }
}
else{
  if ($settings.IISAuthentication -eq "workflowgen"){
    $message = "10. No Windows account need to be created for this authentication mode, however you will be logging in with: " + $settings.WFGAdminUsername
    $message | Out-File -Append log.txt
    Write-Host $message
    $message = "   And the temporary password (which you should immediately change): Passw0rd! "
    $message | Out-File -Append log.txt
    Write-Host $message
  }
}

if(!(Test-Path("IIS:\AppPools\"+$settings.IISAppPoolName))){
  $message = "11. Creating IIS App Pool for WorkflowGen..."
  $message | Out-File -Append log.txt
  Write-Host -NoNewLine $message
  New-WebAppPool -Name $settings.IISAppPoolName | Out-File -Append log.txt
  Set-ItemProperty -Path ("IIS:\AppPools\"+$settings.IISAppPoolName) -Name "managedRuntimeVersion" -Value $constants.WFGIISAppPoolDotNetVersion
  Write-Host "`r$message Done."
}
else{
  $message = "11. Specified IIS App Pool already exists."
  $message | Out-File -Append log.txt
  Write-Host $message
}
$currentWebsite = (Get-Website -Name $settings.IISSiteName)
if($currentWebsite -eq $null){
  $message = "12. Creating IIS Website for WorkflowGen..."
  $message | Out-File -Append log.txt
  Write-Host -NoNewLine $message
  try{
    New-IISSite -Name ($settings.IISSiteName) -PhysicalPath $WFGWebAppPathRoot -BindingInformation ($settings.IISBinding+":"+$settings.IISPort+":") | Out-File -Append log.txt
  }
  catch{
    $_ | Out-File -Append log.txt
    New-Website -Name ($settings.IISSiteName) -PhysicalPath $WFGWebAppPathRoot -Port ($settings.IISPort)  | Out-File -Append log.txt
  }
  Write-Host "Done."
}
else{
  $message = "12. Specified IIS Website already exists."
  $message | Out-File -Append log.txt
  Write-Host $message
  if ($currentWebsite.Bindings.Collection.bindingInformation.Split(":")[1] -ne $setting.IISPort){
    Set-WebBinding -Name $settings.IISSiteName -BindingInformation $currentWebsite.Bindings.Collection.bindingInformation -PropertyName Port -Value $settings.IISPort
  }
}

#Seting the permissions
$message = "13. Granting permissions for WorkflowGen files and folders..."
$message | Out-File -Append log.txt
Write-Host -NoNewLine $message
icacls $settings.WebAppPath /T /Q /inheritance:e /grant:r 'Users:(OI)(CI)F' | Out-File -Append log.txt
icacls $settings.WebAppPath /T /Q /inheritance:e /grant:r ('IIS AppPool\'+$settings.IISAppPoolName+':(OI)(CI)F') | Out-File -Append log.txt
Write-Host "`r$message Done."

$message = "14. Converting WorkflowGen physical folders to web applications..."
$message | Out-File -Append log.txt
Write-Host -NoNewLine $message
if((Get-WebApplication -Site $settings.IISSiteName -Name $WFGWebAppPathLeaf) -eq $null){
  ConvertTo-WebApplication ("IIS:\Sites\"+$settings.IISSiteName+"\"+$WFGWebAppPathLeaf) -ApplicationPool $settings.IISAppPoolName | Out-File -Append log.txt
}
if((Get-WebApplication -Site $settings.IISSiteName -Name ($WFGWebAppPathLeaf+"/WfApps/WebForms")) -eq $null){
  ConvertTo-WebApplication ("IIS:\Sites\"+$settings.IISSiteName+"\"+$WFGWebAppPathLeaf+"\WfApps\WebForms") -ApplicationPool $settings.IISAppPoolName | Out-File -Append log.txt
}
if((Get-WebApplication -Site $settings.IISSiteName -Name ($WFGWebAppPathLeaf+"/ws")) -eq $null){
  ConvertTo-WebApplication ("IIS:\Sites\"+$settings.IISSiteName+"\"+$WFGWebAppPathLeaf+"\ws") -ApplicationPool $settings.IISAppPoolName | Out-File -Append log.txt
}
if((Get-WebApplication -Site $settings.IISSiteName -Name ($WFGWebAppPathLeaf+"/graphql")) -eq $null){
  ConvertTo-WebApplication ("IIS:\Sites\"+$settings.IISSiteName+"\"+$WFGWebAppPathLeaf+"\graphql") -ApplicationPool $settings.IISAppPoolName | Out-File -Append log.txt
}
if((Get-WebApplication -Site $settings.IISSiteName -Name ($WFGWebAppPathLeaf+"/hooks")) -eq $null){
  ConvertTo-WebApplication ("IIS:\Sites\"+$settings.IISSiteName+"\"+$WFGWebAppPathLeaf+"\hooks") -ApplicationPool $settings.IISAppPoolName | Out-File -Append log.txt
}
Write-Host "`r$message Done."

#Setting the authentication type of the website
$message = "15. Configuring IIS authentication..."
$message | Out-File -Append log.txt
Write-Host -NoNewLine $message

Set-WebConfiguration //System.webServer/security/authentication/windowsAuthentication -metadata overrideMode -value Allow -PSPath IIS:/
Set-WebConfiguration //System.webServer/security/authentication/basicAuthentication -metadata overrideMode -value Allow -PSPath IIS:/
Set-WebConfiguration //System.webServer/security/authentication/anonymousAuthentication -metadata overrideMode -value Allow -PSPath IIS:/
if($settings.IISAuthentication -eq "workflowgen")
{
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/windowsAuthentication -name enabled -value false -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/basicAuthentication -name enabled -value false -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/anonymousAuthentication -name enabled -value true -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
}
elseif($settings.IISAuthentication -eq "basic")
{
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/windowsAuthentication -name enabled -value false -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/basicAuthentication -name enabled -value true -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/anonymousAuthentication -name enabled -value false -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
}
else{
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/windowsAuthentication -name enabled -value true -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/basicAuthentication -name enabled -value false -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
  Set-WebConfigurationProperty -filter /system.webServer/security/authentication/anonymousAuthentication -name enabled -value false -PSPath ("IIS:/Sites/"+$settings.IISSiteName)
}
Write-Host "`r$message Done."
